//notation: js file can only use this kind of comments
//since comments will cause error when use in webview.loadurl,
//comments will be remove by java use regexp
(function() {
    if (window.qwapi) {
        return;
    }

    var messagingIframe;
    var sendMessageQueue = [];
    var receiveMessageQueue = [];
    var messageHandlers = {};

    var CUSTOM_PROTOCOL_SCHEME = 'bridge';
    var QUEUE_HAS_MESSAGE = '__QUEUE_MESSAGE__/';

    var responseCallbacks = {};
    var uniqueId = 1;

    function _createQueueReadyIframe(doc) {
        messagingIframe = doc.createElement('iframe');
        messagingIframe.style.display = 'none';
        doc.documentElement.appendChild(messagingIframe);
    }

    //set default messageHandler
    function init(messageHandler) {
        if (qwapi._messageHandler) {
            throw new Error('qwapi.init called twice');
        }
        qwapi._messageHandler = messageHandler;
        var receivedMessages = receiveMessageQueue;
        receiveMessageQueue = null;
        for (var i = 0; i < receivedMessages.length; i++) {
            _dispatchMessageFromNative(receivedMessages[i]);
        }
    }

    function send(data, responseCallback) {
        _doSend({
            data: data
        },
        responseCallback);
    }

    function registerHandler(handlerName, handler) {
        messageHandlers[handlerName] = handler;
    }

    function callHandler(handlerName, data, responseCallback) {
        _doSend({
            handlerName: handlerName,
            data: data
        },
        responseCallback);
    }

    //sendMessage add message, 触发native处理 sendMessage
    function _doSend(message, responseCallback) {
        if (responseCallback) {
            var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
            responseCallbacks[callbackId] = responseCallback;
            message.callbackId = callbackId;
        }

        sendMessageQueue.push(message);
        messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://' + QUEUE_HAS_MESSAGE;
    }

    // 提供给native调用,该函数作用:获取sendMessageQueue返回给native,由于android不能直接获取返回的内容,所以使用url shouldOverrideUrlLoading 的方式返回内容
    function _fetchQueue() {
        var messageQueueString = JSON.stringify(sendMessageQueue);
        sendMessageQueue = [];
        //android can't read directly the return data, so we can reload iframe src to communicate with java
        messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://return/_fetchQueue/' + encodeURIComponent(messageQueueString);
    }

    //提供给native使用,
    function _dispatchMessageFromNative(messageJSON) {
        setTimeout(function() {
            var message = JSON.parse(messageJSON);
            var responseCallback;
            //java call finished, now need to call js callback function
            if (message.responseId) {
                responseCallback = responseCallbacks[message.responseId];
                if (!responseCallback) {
                    return;
                }
                responseCallback(message.responseData);
                delete responseCallbacks[message.responseId];
            } else {
                //直接发送
                if (message.callbackId) {
                    var callbackResponseId = message.callbackId;
                    responseCallback = function(responseData) {
                        _doSend({
                            responseId: callbackResponseId,
                            responseData: responseData
                        });
                    };
                }

                var handler = qwapi._messageHandler;
                if (message.handlerName) {
                    handler = messageHandlers[message.handlerName];
                }
                //查找指定handler
                try {
                    handler(message.data, responseCallback);
                } catch(exception) {
                    if (typeof console != 'undefined') {
                        console.log("qwapi: WARNING: javascript handler threw.", message, exception);
                    }
                }
            }
        });
    }

    //提供给native调用,receiveMessageQueue 在会在页面加载完后赋值为null,所以
    function _handleMessageFromNative(messageJSON) {
        if (receiveMessageQueue && receiveMessageQueue.length > 0) {
            receiveMessageQueue.push(messageJSON);
        } else {
            _dispatchMessageFromNative(messageJSON);
        }
    }
    function action(data, responseCallback) {
        var len = arguments.length;
        if (1 == len) {
            _doSend({
                handlerName: 'action',
                data: data
            },
        function (responseData) { });
        }
        else {
            _doSend({
                handlerName: 'action',
                data: data
            },
         responseCallback);
        }

    }

    function getVars(data, responseCallback) {
        _doSend({
            handlerName: 'getVars',
            data: data
        },
        responseCallback);
    }

    function reply(data) {
            _doSend({
                handlerName: 'reply',
                data: data
            },
            function(responseData) {});
        }

    function share(data) {
            _doSend({
                handlerName: 'share',
                data: data
            },
            function(responseData) {});
        }
    var subscribes = {};
    function subscribe(data, callback) {
        var listeners = subscribes[data];
        if (listeners == undefined) {
            listeners = [];
            listeners.push(callback);
            subscribes[data] = listeners;
            _doSend({
                handlerName: 'subscribe',
                data: data
            },
            function(responseData) {});
        } else {
            var exists = false;
            for (var i = 0; i < listeners.length; i++) {
                var listener = listeners[i];
                if (listener == callback) {
                    exists = true;
                    break;
                }
            }
            if (!exists) {
                subscribes[data].push(callback);
            }
        }
    }

    function unSubscribe() {
        var len = arguments.length;
        if (1 == len) {
            var data = arguments[0];
            unSubscribeAll(data);
        } else if (2 == len) {
            var data = arguments[0];
            var callback = arguments[1];
            unSubscribeCallback(data, callback);
        }
    }

    function unSubscribeAll(data) {
        var listeners = subscribes[data];
        if (listeners != undefined) {
            subscribes[data] = undefined;
            _doSend({
                handlerName: 'unSubscribe',
                data: data
            },
            function(responseData) {});
        }
    }
    function unSubscribeCallback(data, callback) {
        if (subscribes[data] != undefined) {
            var send = false;
            var index = subscribes[data].indexOf(callback);
            if (index != -1) {
                subscribes[data].splice(index, 1);
            }
            var send = subscribes[data].length > 0 ? false: true;
            if (send) {
                _doSend({
                    handlerName: 'unSubscribe',
                    data: data
                },
                function(responseData) {});
            }
        }
    }
    function isString(obj) {
        return Object.prototype.toString.call(obj) === "[object String]";
    }
    registerHandler("onFilter",
    function(data, responseCallback) {
        if (isString(data)) {
            data = JSON.parse(data);
        }
        var listeners = subscribes[data.event];
        if (listeners != undefined) {
            for (var i = 0; i < listeners.length; i++) {
                var listener = listeners[i];
                if (listener != undefined) {
                    listener(data);
                }
            }
        }
    });
    var menu = '';
    function setMenu(data) {
        _doSend({
            handlerName: 'setMenu',
            data: data
        },
        function(responseData) {});
        menu = data;
    }

    function getMenu() {
        var len = arguments.length;
        if (0 == len) {
            return getMenuFromJs();
        } else if (1 == len) {
            var callback = arguments[0];
            getMenuFromNative(callback);
        }
    }
    function getMenuFromNative(responseCallback) {
        _doSend({
            handlerName: 'getMenu',
            data: '{}'
        },
        responseCallback);
    }
    function getMenuFromJs() {
        return menu;
    }
    function adjustTextSize() {
        _doSend({
            handlerName: 'adjustTextSize',
            data: '{}'
        },
        function(responseData) {});
    }

	function setTextSize(adjust) {
    if (navigator.userAgent.match(/iPhone|IOS/i)) {
        adjust = adjust * 100 + "%";
        document.body.style.WebkitTextSizeAdjust = adjust;
        return;
    }
    function getElementsNewFont(tagname, adjust) {
        var arr = [];
        if (tagname.constructor == Array) {
            for (var i = 0, l = tagname.length; i < l; i++) {
                arr = arr.concat(getElementsNewFont(tagname[i], adjust));
            }
        }
        else{
            var elems = document.getElementsByTagName(tagname);
            if (elems && elems.length) {
                for (var i = 0, l = elems.length; i < l; i++) {
                    var ele = elems[i];
                    var currentSize = window.getComputedStyle(ele).fontSize || ele.currentStyle.fontSize;
                    if (!currentSize) {
                        continue;
                    }
                    var originalSize = ele.getAttribute("original-size");
                    if (!originalSize) {
                        ele.setAttribute("original-size", currentSize);
                        originalSize = currentSize;
                    }
                    originalSize = originalSize.match(/(\d+\.?\d*)(.*)/);
                    var toSetSize = originalSize[1] * adjust + originalSize[2];
                    //ele.style.fontSize = toSetSize;
                    arr.push([ele,toSetSize]);
                    //console.log(tagname + "[" + i + "]:", currentSize, "to", toSetSize);
                }
            }
        }
        return arr;
    }
    var arr = getElementsNewFont(["body","h1","h2","h3","h4","h5","h6","a","span","p","div","dd","dl","td","em","font","b","article","ol","li","time"], adjust);
    for (var i = 0, l = arr.length; i < l; i++) {
        arr[i][0].style.fontSize = arr[i][1];
    }
}

    function copyUrl() {
        _doSend({
            handlerName: 'copyUrl',
            data: '{}'
        },
        function(responseData) {});
    }
    function openExternal() {
        var len = arguments.length;
        if (0 == len) {
            openExternalParams('{}');
        } else if (1 == len) {
            var data = arguments[0];
            openExternalParams(data);
        }
    }
    function openExternalParams(data) {
        _doSend({
            handlerName: 'openExternal',
            data: data
        },
        function(responseData) {});
    }

    function express(data, responseCallback) {
        _doSend({
                handlerName: 'express',
                data: data
            },
            responseCallback);
    }

    function route(data, responseCallback) {
            _doSend({
                    handlerName: 'route',
                    data: data
                },
                function(responseData) {});
        }

    function reload() {
        _doSend({
            handlerName: 'reload',
            data: '{}'
        },
        function(responseData) {});
    }
    var qwapi = window.qwapi = {
        init: init,
        send: send,
        action: action,
        getVars: getVars,
        subscribe: subscribe,
        route:route,
        reply:reply,
        share:share,
        unSubscribe: unSubscribe,
        setMenu: setMenu,
        express:express,
        getMenu: getMenu,
        adjustTextSize: adjustTextSize,
        setTextSize: setTextSize,
        copyUrl: copyUrl,
        openExternal: openExternal,
        reload: reload,
        registerHandler: registerHandler,
        callHandler: callHandler,
        _fetchQueue: _fetchQueue,
        _handleMessageFromNative: _handleMessageFromNative
    };

    var doc = document;
    _createQueueReadyIframe(doc);
    var readyEvent = doc.createEvent('Events');
    readyEvent.initEvent('qwapiReady');
    readyEvent.bridge = qwapi;
    doc.dispatchEvent(readyEvent);
})();